"""Import specific class that will use."""
from .drone_deploy import DroneDeploy
from .aerialcactus import AerialCactus
from .eurosat import EuroSat
from .resisc45 import RESISC45
from .ucmercedland import UCMercedLand
from .l8sparcs import L8SPARCS
from .deepsat import DeepSat
from .landcover import LandCover
from .cowc import COWC

__all__ = ['DroneDeploy', 'AerialCactus', 'RESISC45',
           'UCMercedLand', 'EuroSat', 'L8SPARCS', 'DeepSat', 'LandCover', 'COWC']



import os
import shutil
import posixpath
import numpy as np
import pandas as pd
import glob
import torch
from torch.utils.data import Dataset
from torchvision.transforms import Resize
from PIL import Image
from .utils import _urlretrieve, _load_img

class L8SPARCS():
    """Landsat 8 SPARCS Cloud. 
    <https://www.usgs.gov/core-science-systems/nli/landsat/spatial-procedures-automated-removal-cloud-and-shadow-sparcs>

    Download: <https://landsat.usgs.gov/cloud-validation/sparcs/l8cloudmasks.zip>

    Args:
        root (string): Root directory of dataset.
    """
    
    mirrors = "https://landsat.usgs.gov/cloud-validation/sparcs/"
    resources = "l8cloudmasks.zip"
    
    def __init__(self,
                root: str,
                data_mode: str = 'sending'):
        
        self.root = root
        self.data_mode = data_mode

        if not self._check_exists():
            self.download()
            self.extract_file()

        self.img_labels = self.get_path_and_label()

    def _check_exists(self) -> None:
        self.data_path = os.path.join(
            self.root, self.data_mode)
        return os.path.exists(self.data_path)

    def download(self):
       """Download file"""
       file_url = posixpath.join(self.mirrors, self.resources)
       _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract the .zip file"""
        shutil.unpack_archive(os.path.join(self.root, self.resources), self.root)
        os.remove(os.path.join(self.root, self.resources))


    def get_path_and_label(self):
        """Get the path of the images and labels (masks) in a dataframe"""
        image_path = []
        label = []

        for image in glob.glob(os.path.join(self.root,self.data_mode,'*_photo.png')):
            image_path.append(image)
        
        for mask in glob.glob(os.path.join(self.root,self.data_mode,'*_mask.png')):
            label.append(mask)

        df = pd.DataFrame({'image': sorted(image_path), 'label': sorted(label)})

        return df
    
    def __getitem__(self, idx):
        """Return a tensor image and its tensor mask"""
        img_path = self.img_labels.iloc[idx, 0]
        mask_path = self.img_labels.iloc[idx, 1]

        image = _load_img(img_path)
        image = np.array(image)
        image = torch.from_numpy(image)
        
        mask = _load_img(mask_path)
        mask = np.array(mask)
        mask = torch.from_numpy(mask)
        
        sample = (image, mask)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)


"""Class for Deepsat - Scene Classification."""
import os
import scipy.io as sio
import torch
import requests
import gdown
import sys

from torch.utils.data import Dataset


class DeepSat(Dataset):
    """DeepSat Dataset.

    Args:
        root (string): Root directory of dataset.
        dataset_type (string, optional): Choose dataset type ['SAT-4', 'SAT-6'].
        download (bool, optional): If true, downloads the dataset from the internet and
            puts it in root directory. If dataset is already downloaded, it is not
            downloaded again.
        data_mode (int): 0 for train data, and 1 for test data.
    """

    resources = {
        'SAT-4_and_SAT-6_datasets': 'https://drive.google.com/uc?id=0B0Fef71_vt3PUkZ4YVZ5WWNvZWs&export=download'}
    dataset_types = ['SAT-4', 'SAT-6']

    def __init__(self, root: str, dataset_type='SAT-4', download: bool = False, data_mode: int = 0):
        self.root = root
        self.dataset_type = dataset_type
        self.data_mode = data_mode
        self.folder_pth = os.path.join(
            self.root, list(self.resources.keys())[0])
        self.filename = list(self.resources.keys())[0] + '.tar.gz'

        if download and self._check_exists():
            print(f'zipfile "{self.filename}" already exists.')

        if download and not self._check_exists():
            self.download()

        dataset = self.load_dataset()
        self.choose_data_mode(dataset)

    def download(self):
        """Download dataset and extract it"""

        self.root = os.path.expanduser(self.root)
        print("Download dataset...")

        gdown.download(self.resources['SAT-4_and_SAT-6_datasets'],
                       os.path.join(self.root, self.filename), quiet=False)

        if os.path.exists(self.folder_pth):
            print(f'file {self.folder_pth} already exists')
        else:
            os.mkdir(self.folder_pth)
            print(f'Extracting file {self.filename}')
            os.system(
                f'tar -xvf {os.path.join(self.root, self.filename)} -C {self.folder_pth}')
            os.system(f'mv {self.folder_pth} {self.root}')
            print("Extracting file success !")

    def _check_exists(self) -> bool:
        if self.dataset_type not in self.dataset_types:
            print(f"Unknown dataset {self.dataset_type}")
            print(f"Available dataset : {self.dataset_types}")
            sys.exit(0)

        if os.path.exists(self.filename):
            return True
        else:
            return False

    def load_dataset(self):
        filename = {'SAT-4': 'sat-4-full.mat', 'SAT-6': 'sat-6-full.mat'}
        dataset = sio.loadmat(os.path.join(
            self.folder_pth, filename[self.dataset_type]))
        return dataset

    def choose_data_mode(self, dataset):
        if self.data_mode == 0:
            x_type, y_type = 'train_x', 'train_y'
        elif self.data_mode == 1:
            x_type, y_type = 'test_x', 'test_y'

        self.x, self.y = dataset[x_type], dataset[y_type]
        self.annot = dataset['annotations']

    def __len__(self):
        return self.x.shape[3]

    def __getitem__(self, idx):
        img = self.x[:, :, :, idx]
        label = self.y[:, idx]
        tensor_image = torch.from_numpy(img)
        tensor_label = torch.from_numpy(label)
        return tensor_image, tensor_label



file_mapping_counting = {
    'Columbus_CSUAV_AFRL': 'COWC_Counting_Columbus_CSUAV_AFRL.tbz',
    'Potsdam_ISPRS': 'COWC_Counting_Potsdam_ISPRS.tbz',
    'Selwyn_LINZ': 'COWC_Counting_Selwyn_LINZ.tbz',
    'Toronto_ISPRS': 'COWC_Counting_Toronto_ISPRS.tbz',
    'Utah_AGRC': 'COWC_Counting_Utah_AGRC.tbz',
    'Vaihingen_ISPRS': 'COWC_Counting_Vaihingen_ISPRS.tbz'
}

file_mapping_detection = {
    'Columbus_CSUAV_AFRL': 'COWC_Detection_Columbus_CSUAV_AFRL.tbz',
    'Potsdam_ISPRS': 'COWC_Detection_Potsdam_ISPRS.tbz',
    'Selwyn_LINZ': 'COWC_Detection_Selwyn_LINZ.tbz',
    'Toronto_ISPRS': 'COWC_Detection_Toronto_ISPRS.tbz',
    'Utah_AGRC': 'COWC_Detection_Utah_AGRC.tbz',
    'Vaihingen_ISPRS': 'COWC_Detection_Vaihingen_ISPRS.tbz'
}


CLASS_ENC = {
    'airplane': 0,
    'airport': 1,
    'baseball_diamond': 2,
    'basketball_court': 3,
    'beach': 4,
    'bridge': 5,
    'chaparral': 6,
    'church': 7,
    'circular_farmland': 8,
    'cloud': 9,
    'commercial_area': 10,
    'dense_residential': 11,
    'desert': 12,
    'forest': 13,
    'freeway': 14,
    'golf_course': 15,
    'ground_track_field': 16,
    'harbor': 17,
    'industrial_area': 18,
    'intersection': 19,
    'island': 20,
    'lake': 21,
    'meadow': 22,
    'medium_residential': 23,
    'mobile_home_park': 24,
    'mountain': 25,
    'overpass': 26,
    'palace': 27,
    'parking_lot': 28,
    'railway': 29,
    'railway_station': 30,
    'rectangular_farmland': 31,
    'river': 32,
    'roundabout': 33,
    'runway': 34,
    'sea_ice': 35,
    'ship': 36,
    'snowberg': 37,
    'sparse_residential': 38,
    'stadium': 39,
    'storage_tank': 40,
    'tennis_court': 41,
    'terrace': 42,
    'thermal_power_station': 43,
    'wetland': 44
}

CLASS_DEC = {y:x for x,y in CLASS_ENC.items()}


import os
import shutil
import posixpath
import numpy as np
import pandas as pd
import torch
from torch.utils.data import Dataset
from torchvision.transforms import Resize
from .utils import _urlretrieve, _load_img


class UCMercedLand(Dataset):
    """UC Merced Land Use Dataset.
    <http://weegee.vision.ucmerced.edu/datasets/UCMerced_LandUse.zip>
    """

    mirrors = "http://weegee.vision.ucmerced.edu/datasets/"
    resources = "UCMerced_LandUse.zip"

    def __init__(self,
                 root: str,
                 data_mode: str = 'Images',
                 transform=Resize((256, 256)),
                 target_transform=None):

        self.root = root
        self.data_mode = data_mode
        self.transform = transform
        self.target_transform = target_transform

        if not self._check_exists():
            self.download()
            self.extract_file()

        self.img_labels = self.get_path_and_label()

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        label = self.img_labels.iloc[idx, 1]
        image = _load_img(img_path)
        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            label = self.target_transform(label)
        image = np.array(image)
        image = torch.from_numpy(image)
        sample = (image, label)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)

    def get_path_and_label(self):
        """Return dataframe type consist of image path and corresponding label."""
        classes = {'agricultural': 0,
                   'airplane': 1,
                   'baseballdiamond': 2,
                   'beach': 3,
                   'buildings': 4,
                   'chaparral': 5,
                   'denseresidential': 6,
                   'forest': 7,
                   'freeway': 8,
                   'golfcourse': 9,
                   'harbor': 10,
                   'intersection': 11,
                   'mediumresidential': 12,
                   'mobilehomepark': 13,
                   'overpass': 14,
                   'parkinglot': 15,
                   'river': 16,
                   'runway': 17,
                   'sparseresidential': 18,
                   'storagetanks': 19,
                   'tenniscourt': 20}
        image_path = []
        label = []
        for cat, enc in classes.items():
            cat_path = os.path.join(
                self.root, 'UCMerced_LandUse', self.data_mode, cat)
            cat_image = [os.path.join(cat_path, path)
                         for path in os.listdir(cat_path)]
            cat_label = [enc] * len(cat_image)
            image_path += cat_image
            label += cat_label
        df = pd.DataFrame({'image': image_path, 'label': label})

        return df

    def _check_exists(self):
        self.data_path = os.path.join(
            self.root, "UCMerced_LandUse", "Images")

        return os.path.exists(os.path.join(self.data_path, "agricultural")) and \
            os.path.exists(os.path.join(self.data_path, "airplane")) and \
            os.path.exists(os.path.join(self.data_path, "baseballdiamond")) and \
            os.path.exists(os.path.join(self.data_path, "beach")) and \
            os.path.exists(os.path.join(self.data_path, "buildings")) and \
            os.path.exists(os.path.join(self.data_path, "chaparral")) and \
            os.path.exists(os.path.join(self.data_path, "denseresidential")) and \
            os.path.exists(os.path.join(self.data_path, "forest")) and \
            os.path.exists(os.path.join(self.data_path, "freeway")) and \
            os.path.exists(os.path.join(self.data_path, "golfcourse")) and \
            os.path.exists(os.path.join(self.data_path, "harbor")) and \
            os.path.exists(os.path.join(self.data_path, "intersection")) and \
            os.path.exists(os.path.join(self.data_path, "mediumresidential")) and \
            os.path.exists(os.path.join(self.data_path, "mobilehomepark")) and \
            os.path.exists(os.path.join(self.data_path, "overpass")) and \
            os.path.exists(os.path.join(self.data_path, "parkinglot")) and \
            os.path.exists(os.path.join(self.data_path, "river")) and \
            os.path.exists(os.path.join(self.data_path, "runway")) and \
            os.path.exists(os.path.join(self.data_path, "sparseresidential")) and \
            os.path.exists(os.path.join(self.data_path, "storagetanks")) and \
            os.path.exists(os.path.join(self.data_path, "tenniscourt"))

    def download(self):
        """download and extract file."""
        file_url = posixpath.join(self.mirrors, self.resources)
        _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract file from compressed."""
#         path_destination = os.path.join(
#             self.root, self.resources.replace(".zip", ""))
#         os.makedirs(path_destination, exist_ok=True)
        shutil.unpack_archive(os.path.join(
            self.root, self.resources), self.root)
        os.remove(os.path.join(self.root, self.resources))



"""Class for Drone Deploy - Semantic Segmentation."""
import sys
import os
import numpy as np
import random
import cv2
import torch
from ..constants.DroneDeploy.config import train_ids, val_ids, test_ids, LABELMAP, INV_LABELMAP
from .utils import _urlretrieve

from PIL import Image
from torch.utils.data import Dataset


class DroneDeploy():
    """Drone Deploy Semantic Dataset.

    Args:
        root (string): Root directory of dataset.
        dataset_type (string, optional): Choose dataset type.
        download (bool, optional): If true, downloads the dataset from the internet and
            puts it in root directory. If dataset is already downloaded, it is not
            downloaded again.
        data_mode (int): 0 for train data, 1 for validation data, and 2 for testing data
    """

    resources = {
        'dataset-sample': 'https://dl.dropboxusercontent.com/s/h8a8kev0rktf4kq/dataset-sample.tar.gz?dl=0',
        'dataset-medium': 'https://dl.dropboxusercontent.com/s/r0dj9mhyv4bgbme/dataset-medium.tar.gz?dl=0'
    }

    def __init__(self, root: str, dataset_type='dataset-sample', download: bool = False, data_mode: int = 0):
        self.root = root
        self.dataset_type = dataset_type
        self.filename = f'{dataset_type}.tar.gz'
        self.filepath = os.path.join(self.root, self.filename)
        self.data_mode = data_mode
        self.label_path = f'{dataset_type}/label-chips'
        self.image_path = f'{dataset_type}/image-chips'

        if download and self._check_exists():
            print(f'zipfile "{self.filepath}" already exists.')

        if download and not self._check_exists():
            self.download()

        self.load_dataset()

    def download(self):
        """Download a dataset, extract it and create the tiles."""
        print(f'Downloading "{self.dataset_type}"')
        self.root = os.path.expanduser(self.root)
        fpath = os.path.join(self.root, self.filename)
        _urlretrieve(self.resources[self.dataset_type], fpath)

        if not os.path.exists(os.path.join(self.root, self.dataset_type)):
            print(f'Extracting "{self.filepath}"')
            os.system(f'tar -xvf {self.filepath}')
            os.system(f'mv {self.dataset_type} {self.root}')
        else:
            print(f'Folder "{self.dataset_type}" already exists.')

        image_chips = f'{self.dataset_type}/image-chips'
        label_chips = f'{self.dataset_type}/label-chips'

        if not os.path.exists(image_chips):
            os.mkdir(os.path.join(self.root, image_chips))
        if not os.path.exists(label_chips):
            os.mkdir(os.path.join(self.root, label_chips))

        run(os.path.join(self.root, self.dataset_type))

    def _check_exists(self) -> bool:
        if self.dataset_type not in self.resources.keys():
            print(f"Unknown dataset {self.dataset_type}")
            print(f"Available dataset : {self.resources.keys()}")
            sys.exit(0)

        if os.path.exists(self.filepath):
            return True
        else:
            return False

    def load_dataset(self):
        if self.data_mode == 0:
            list_chip = 'train.txt'
        elif self.data_mode == 1:
            list_chip = 'valid.txt'
        elif self.data_mode == 2:
            list_chip = 'test.txt'

        files = [f'{os.path.join(self.root, self.dataset_type)}/image-chips/{fname}'
                 for fname in load_lines(os.path.join(self.root, self.dataset_type, list_chip))]
        self.image_files = files

    def __len__(self):
        return len(self.image_files)

    def __getitem__(self, idx):
        image_file = self.image_files[idx]
        label_file = image_file.replace(self.image_path, self.label_path)

        image = load_img(image_file)
        label = mask_to_classes(load_img(label_file))

        tensor_image = torch.from_numpy(np.array(image))
        tensor_label = torch.from_numpy(np.array(label))
        return tensor_image, tensor_label

    def on_epoch_end(self):
        random.shuffle(self.image_files)


def load_lines(fname):
    with open(fname, 'r') as f:
        return [line.strip() for line in f.readlines()]


def load_img(fname):
    return np.array(Image.open(fname))


def mask_to_classes(mask):
    return to_categorical(mask[:, :, 0], 6)


def to_categorical(y, num_classes=None, dtype='float32'):
    """Converts a class vector (integers) to binary class matrix.
    E.g. for use with categorical_crossentropy.
    Args:
        y: class vector to be converted into a matrix
            (integers from 0 to num_classes).
        num_classes: total number of classes. If `None`, this would be inferred
          as the (largest number in `y`) + 1.
        dtype: The data type expected by the input. Default: `'float32'`.
    Returns:
        A binary matrix representation of the input. The classes axis is placed
        last.
    Raises:
        Value Error: If input contains string value
    """
    y = np.array(y, dtype='int')
    input_shape = y.shape
    if input_shape and input_shape[-1] == 1 and len(input_shape) > 1:
        input_shape = tuple(input_shape[:-1])
    y = y.ravel()
    if not num_classes:
        num_classes = np.max(y) + 1
    n = y.shape[0]
    categorical = np.zeros((n, num_classes), dtype=dtype)
    categorical[np.arange(n), y] = 1
    output_shape = input_shape + (num_classes,)
    categorical = np.reshape(categorical, output_shape)
    return categorical


def get_split(scene):
    if scene in train_ids:
        return "train.txt"
    if scene in val_ids:
        return 'valid.txt'
    if scene in test_ids:
        return 'test.txt'


def color2class(orthochip, img):
    ret = np.zeros((img.shape[0], img.shape[1]), dtype='uint8')
    ret = np.dstack([ret, ret, ret])
    colors = np.unique(img.reshape(-1, img.shape[2]), axis=0)

    # Skip any chips that would contain magenta (IGNORE) pixels
    seen_colors = set([tuple(color) for color in colors])
    IGNORE_COLOR = LABELMAP[0]
    if IGNORE_COLOR in seen_colors:
        return None, None

    for color in colors:
        locs = np.where((img[:, :, 0] == color[0]) & (
            img[:, :, 1] == color[1]) & (img[:, :, 2] == color[2]))
        ret[locs[0], locs[1], :] = INV_LABELMAP[tuple(color)] - 1

    return orthochip, ret


def image2tile(prefix, scene, dataset, orthofile, elevafile, labelfile, windowx,
               windowy, stridex, stridey):

    ortho = cv2.imread(orthofile)
    label = cv2.imread(labelfile)

    assert(ortho.shape[0] == label.shape[0])
    assert(ortho.shape[1] == label.shape[1])

    shape = ortho.shape
    xsize = shape[1]
    ysize = shape[0]
    print(
        f"converting {dataset} image {orthofile} {xsize}x{ysize} to chips ...")

    counter = 0
    for xi in range(0, shape[1] - windowx, stridex):
        for yi in range(0, shape[0] - windowy, stridey):
            orthochip = ortho[yi:yi+windowy, xi:xi+windowx, :]
            labelchip = label[yi:yi+windowy, xi:xi+windowx, :]

            orthochip, classchip = color2class(orthochip, labelchip)

            if classchip is None:
                continue

            orthochip_filename = os.path.join(prefix, 'image-chips',
                                              scene + '-' + str(counter).zfill(6) + '.png')
            labelchip_filename = os.path.join(prefix, 'label-chips',
                                              scene + '-' + str(counter).zfill(6) + '.png')

            with open(f"{prefix}/{dataset}", mode='a') as fd:
                fd.write(scene + '-' + str(counter).zfill(6) + '.png\n')

            cv2.imwrite(orthochip_filename, orthochip)
            cv2.imwrite(labelchip_filename, classchip)
            counter += 1


def run(prefix, size=300, stride=300):
    lines = [line for line in open(f'{prefix}/index.csv')]
    print("converting images to chips - this may take a few minutes but only needs to be done once.")

    for lineno, line in enumerate(lines):
        line = line.strip().split(' ')
        scene = line[1]
        dataset = get_split(scene)

        orthofile = os.path.join(prefix, 'images',     scene + '-ortho.tif')
        elevafile = os.path.join(prefix, 'elevations', scene + '-elev.tif')
        labelfile = os.path.join(prefix, 'labels',     scene + '-label.png')

        if os.path.exists(orthofile) and os.path.exists(labelfile):
            image2tile(prefix, scene, dataset, orthofile, elevafile, labelfile,
                       windowx=size, windowy=size, stridex=stride, stridey=stride)



import os
import shutil
import posixpath
import numpy as np
import pandas as pd
import torch
from torch.utils.data import Dataset
from torchvision.transforms import Resize
from PIL import Image
from .utils import _urlretrieve, _load_img


class EuroSat():
    """EuroSat Land Cover Categories. 
    `Download EuroSat RGB <http://madm.dfki.de/files/sentinel>`

    Args:
        root (string): Root directory of dataset.
    """
    
    mirrors = "http://madm.dfki.de/files/sentinel"
    resources = "EuroSAT.zip"

    def __init__(self,
                root: str,
                data_mode: str = '2750',
                transform=Resize((64, 64)),
                target_transform=None):
                
        self.root = root
        self.data_mode = data_mode
        self.transform = transform
        self.target_transform = target_transform

        if not self._check_exists():
            self.download()
            self.extract_file()

        self.img_labels = self.get_path_and_label()

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        label = self.img_labels.iloc[idx, 1]
        image = _load_img(img_path)
        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            label = self.target_transform(label)
        image = np.array(image)
        image = torch.from_numpy(image)
        sample = (image, label)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)

    def _check_exists(self) -> None:
        self.data_path = os.path.join(
            self.root, self.data_mode)

        return os.path.exists(os.path.join(self.data_path, "AnnualCrop")) and \
            os.path.exists(os.path.join(self.data_path, "Forest")) and \
            os.path.exists(os.path.join(self.data_path, "HerbaceousVegetation")) and \
            os.path.exists(os.path.join(self.data_path, "Highway")) and \
            os.path.exists(os.path.join(self.data_path, "Industrial")) and \
            os.path.exists(os.path.join(self.data_path, "Pasture")) and \
            os.path.exists(os.path.join(self.data_path, "PermanentCrop")) and \
            os.path.exists(os.path.join(self.data_path, "Residential")) and \
            os.path.exists(os.path.join(self.data_path, "River")) and \
            os.path.exists(os.path.join(self.data_path, "SeaLake"))


    def download(self):
       """Download file"""
       file_url = posixpath.join(self.mirrors, self.resources)
       _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract the .zip file"""
        shutil.unpack_archive(os.path.join(self.root, self.resources), self.root)
        os.remove(os.path.join(self.root, self.resources))

    def get_path_and_label(self):
        """Return dataframe type consist of image path and corresponding label."""
        classes = {"AnnualCrop": 0, \
                    "Forest": 1, \
                    "HerbaceousVegetation": 2, \
                    "Highway": 3, \
                    "Industrial": 4, \
                    "Pasture": 5, \
                    "PermanentCrop": 6, \
                    "Residential": 7, \
                    "River": 8, \
                    "SeaLake": 9}
        image_path = []
        label = []
        for cat, enc in classes.items():
            cat_path = os.path.join(
                self.root, self.data_mode, cat)
            cat_image = [os.path.join(cat_path, path)
                            for path in os.listdir(cat_path)]
            cat_label = [enc] * len(cat_image)
            image_path += cat_image
            label += cat_label
        df = pd.DataFrame({'image': image_path, 'label': label})

        return df


"""Aerial Cactus Dataset from Kaggle."""
import os
import shutil
import posixpath
import numpy as np
import pandas as pd
import torch
from torch.utils.data import Dataset
from torchvision.transforms import Resize
from .utils import _urlretrieve, _load_img


class AerialCactus(Dataset):
    """Aerial Cactus Dataset.
    <https://www.kaggle.com/c/aerial-cactus-identification>
    """

    mirrors = "https://storage.googleapis.com/ossjr"
    resources = "cactus-aerial-photos.zip"

    def __init__(self,
                 root: str,
                 data_mode: str = 'training_set',
                 transform=Resize((32, 32)),
                 target_transform=None):

        self.root = root
        self.data_mode = data_mode
        self.transform = transform
        self.target_transform = target_transform

        if not self._check_exists():
            self.download()
            self.extract_file()

        self.img_labels = self.get_path_and_label()

    def __len__(self):
        return len(self.img_labels)

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        label = self.img_labels.iloc[idx, 1]
        image = _load_img(img_path)
        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            label = self.target_transform(label)
        image = np.array(image)
        image = torch.from_numpy(image)
        sample = (image, label)

        return sample

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)

    def get_path_and_label(self):
        """Return dataframe type consist of image path and corresponding label."""
        classes = {'cactus': 1, 'no_cactus': 0}
        image_path = []
        label = []
        for cat, enc in classes.items():
            cat_path = os.path.join(
                self.root, 'cactus-aerial-photos', self.data_mode, self.data_mode, cat)
            cat_image = [os.path.join(cat_path, path)
                         for path in os.listdir(cat_path)]
            cat_label = [enc] * len(cat_image)
            image_path += cat_image
            label += cat_label
        df = pd.DataFrame({'image': image_path, 'label': label})

        return df

    def _check_exists(self):
        self.train_path = os.path.join(
            self.root, "cactus-aerial-photos", "training_set", "training_set")
        self.test_path = os.path.join(
            self.root, "cactus-aerial-photos", "validation_set", "validation_set")

        return os.path.exists(os.path.join(self.train_path, "cactus")) and \
            os.path.exists(os.path.join(self.train_path, "no_cactus")) and \
            os.path.exists(os.path.join(self.test_path, "cactus")) and \
            os.path.exists(os.path.join(self.test_path, "no_cactus"))

    def download(self):
        """Download and extract file."""
        file_url = posixpath.join(self.mirrors, self.resources)
        _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract file from compressed."""
        shutil.unpack_archive(self.resources, self.root)
        os.remove(os.path.join(self.root, self.resources))



"""Cars Overhead with Context."""
import os
import shutil
import posixpath
import tarfile
import torch
import numpy as np
import pandas as pd
from torch.utils.data import Dataset
from .utils import _urlretrieve, _load_img
from ..constants.COWC.config import file_mapping_counting, \
    file_mapping_detection


class COWC():
    """ Cars Overhead with Context.
    https://gdo152.llnl.gov/cowc/
    """

    mirrors = "https://gdo152.llnl.gov/cowc/download"
    resources = "cowc-everything.txz"

    def __init__(self,
                 root: str,
                 data_mode: str = 'train',
                 task_mode: str = 'counting',
                 transform=None,
                 target_transform=None):
        self.root = root
        self.data_mode = data_mode
        self.task_mode = task_mode
        self.transform = transform
        self.target_transform = target_transform

        if not self._check_exists():
            self.download()
            self.extract_file()

        if self.task_mode == 'counting':
            self.task_path = os.path.join(
                self.root, 'cowc/datasets/patch_sets/counting'
            )
            self.file_mapping = file_mapping_counting
        elif self.task_mode == 'detection':
            self.task_path = os.path.join(
                self.root, 'cowc/datasets/patch_sets/detection'
            )
            self.file_mapping = file_mapping_detection
        else:
            raise ValueError('task_mode not recognized.')

        for filename, compressed in self.file_mapping.items():
            if not self._check_exists_subfile(filename):
                self.extract_subfile(filename, compressed)

        self.img_labels = self.get_path_and_label()

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        label = self.img_labels.iloc[idx, 1]
        folder = img_path.split('/', 1)[0]
        img_path = os.path.join(self.task_path, folder, img_path)
        image = _load_img(img_path)

        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            label = self.target_transform(label)

        image = np.array(image)
        image = torch.from_numpy(image)
        sample = (image, label)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def get_path_and_label(self):
        """Return dataframe type consist of image path
        and corresponding label."""

        if self.task_mode == 'counting':
            if self.data_mode == 'train':
                label_name = 'COWC_train_list_64_class.txt.bz2'
            elif self.data_mode == 'test':
                label_name = 'COWC_test_list_64_class.txt.bz2'
            else:
                raise ValueError('data_mode not recognized.')
        elif self.task_mode == 'detection':
            if self.data_mode == 'train':
                label_name = 'COWC_train_list_detection.txt.bz2'
            elif self.data_mode == 'test':
                label_name = 'COWC_test_list_detection.txt.bz2'
            else:
                raise ValueError('data_mode not recognized.')
        else:
            raise ValueError('task_mode not recognized.')

        label_path = os.path.join(self.task_path, label_name)
        df = pd.read_csv(label_path, sep=' ', header=None)

        return df

    def _check_exists_subfile(self, filename):
        path_to_check = os.path.join(self.task_path, filename)
        return os.path.exists(path_to_check)

    def extract_subfile(self, filename, compressed):
        comp_path = os.path.join(self.task_path, compressed)
        file_path = os.path.join(self.task_path, filename)
        tar = tarfile.open(comp_path)
        tar.extractall(file_path)
        tar.close()

    def _check_exists(self):
        return os.path.exists(os.path.join(self.root, "cowc"))

    def download(self):
        """download file."""
        file_url = posixpath.join(self.mirrors, self.resources)
        _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract file from compressed."""
        shutil.unpack_archive(os.path.join(
            self.root, self.resources), self.root)
        os.remove(os.path.join(self.root, self.resources))



"""RESISC45 Dataset."""
import os
import posixpath
import shutil
import numpy as np
import pandas as pd
import torch
from torch.utils.data import Dataset
from ..constants.RESISC45.config import CLASS_ENC, CLASS_DEC
from .utils import _urlretrieve, _load_img


class RESISC45(Dataset):

    mirrors = "https://storage.googleapis.com/ossjr"
    resources = "NWPU-RESISC45.zip"

    def __init__(self, root, transform=None, target_transform=None):
        self.root = root
        self.transform = transform
        self.target_transform = target_transform
        self.class_enc = CLASS_ENC
        self.class_dec = CLASS_DEC

        if not self._check_exists():
            self.download()
            self.extract_file()

        self.img_labels = self.get_path_and_label()

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        label = self.img_labels.iloc[idx, 1]
        image = _load_img(img_path)
        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            label = self.target_transform(label)
        image = np.array(image)
        image = torch.from_numpy(image)
        sample = (image, label)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)

    def get_path_and_label(self):
        """Return dataframe type consist of image path and corresponding label."""
        DATA_SIZE = 700
        category = os.listdir(os.path.join(self.root, 'NWPU-RESISC45'))
        image_path = []
        label = []
        for cat in category:
            cat_enc = self.class_enc[cat]
            label += [cat_enc] * DATA_SIZE
            for num in range(1, DATA_SIZE+1):
                filename = cat + '_' + str(num).zfill(3) + '.jpg'
                image_path += [os.path.join(self.root,
                                            'NWPU-RESISC45', cat, filename)]
        df = pd.DataFrame({'image': image_path, 'label': label})

        return df

    def _check_exists(self):
        is_exists = os.path.exists(os.path.join(self.root, "NWPU-RESISC45"))

        return is_exists

    def download(self):
        """Download and extract file."""
        file_url = posixpath.join(self.mirrors, self.resources)
        _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def extract_file(self):
        """Extract file from compressed."""
        shutil.unpack_archive(os.path.join(
            self.root, self.resources), f"{self.root}")
        os.remove(os.path.join(self.root, self.resources))



import urllib
import numpy as np
from tqdm import tqdm
from PIL import Image


def _urlretrieve(url: str, filename: str, chunk_size: int = 1024) -> None:
    with open(filename, "wb") as fh:
        with urllib.request.urlopen(urllib.request.Request(url)) as response:
            with tqdm(total=response.length) as pbar:
                for chunk in iter(lambda: response.read(chunk_size), ""):
                    if not chunk:
                        break
                    pbar.update(chunk_size)
                    fh.write(chunk)


def _load_img(fname):
    return Image.open(fname)


from setuptools import setup, find_packages

with open('README.md', encoding='utf8') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    required = f.read().splitlines()

version = '0.0.1'
download_url = f'https://github.com/jakartaresearch/earth-vision/archive/v{version}.tar.gz'

setup(
    name='earth-vision',
    packages=find_packages(),
    version=version,
    license='MIT',
    description='Python library for solving computer vision tasks specifically for satellite imagery',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Jakarta Research Team',
    author_email='researchjair@gmail.com',
    url='https://github.com/jakartaresearch/earth-vision',
    download_url=download_url,
    keywords=['computer-vision', 'pytorch',
              'machine-learning', 'satellite-imagery'],
    install_requires=required,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from earthvision import dataset

try:
    l8biome = dataset.L8Biome()
    print("Your development environment has been setup properly")
except:
    print("Installation failed, please open an issue to let us know!")



import os
import shutil
import posixpath
import numpy as np
import pandas as pd
import torch
from torch.utils.data import Dataset
from torchvision.transforms import Resize
from .utils import _urlretrieve, _load_img

import glob
import cv2


class LandCover(Dataset):

    """
    The LandCover.ai (Land Cover from Aerial Imagery) dataset.
    <https://landcover.ai/download/landcover.ai.v1.zip>
    """

    mirrors = "https://landcover.ai/download"
    resources = "landcover.ai.v1.zip"

    def __init__(self,
                 root: str,
                 data_mode: str = 'Images',
                 transform=Resize((256, 256)),
                 target_transform=Resize((256, 256))):

        self.root = root
        self.data_mode = data_mode
        self.transform = transform
        self.target_transform = target_transform

        if not self._check_exists():
            self.download()
            self.extract_file()
            self.to_chip_img_mask("landcover")

        self.img_labels = self.get_image_path_and_mask_path()

    def __getitem__(self, idx):
        img_path = self.img_labels.iloc[idx, 0]
        mask_path = self.img_labels.iloc[idx, 1]
        image = _load_img(img_path)
        mask = _load_img(mask_path)
        if self.transform:
            image = self.transform(image)
        if self.target_transform:
            mask = self.target_transform(mask)
        image = np.array(image)
        image = torch.from_numpy(image)
        mask = np.array(mask)
        mask = torch.from_numpy(mask)
        sample = (image, mask)

        return sample

    def __len__(self):
        return len(self.img_labels)

    def __iter__(self):
        for index in range(self.__len__()):
            yield self.__getitem__(index)

    def get_image_path_and_mask_path(self):
        """Return dataframe type consist of image path and mask path."""
        image_path = []
        mask_path = []

        img_path = os.path.join(self.root, 'landcover', 'output', 'images')
        msk_path = os.path.join(self.root, 'landcover', 'output', 'masks')

        images_path = [os.path.join(img_path, path)
                       for path in os.listdir(img_path)]
        images_path.sort()
        masks_path = [os.path.join(img_path, path)
                      for path in os.listdir(msk_path)]
        masks_path.sort()

        df = pd.DataFrame({'image': images_path, 'mask': masks_path})
        return df

    def to_chip_img_mask(self, base):

        IMGS_DIR = "./{}/images".format(base)
        MASKS_DIR = "./{}/masks".format(base)
        OUTPUT_DIR = "./{}/output".format(base)
        OUTPUT_IMGS_DIR = "./{}/output/images".format(base)
        OUTPUT_MASKS_DIR = "./{}/output/masks".format(base)

        TARGET_SIZE = 512

        img_paths = glob.glob(os.path.join(IMGS_DIR, "*.tif"))
        mask_paths = glob.glob(os.path.join(MASKS_DIR, "*.tif"))

        img_paths.sort()
        mask_paths.sort()

        # os.makedirs(OUTPUT_DIR)
        os.makedirs(OUTPUT_IMGS_DIR)
        os.makedirs(OUTPUT_MASKS_DIR)
        for i, (img_path, mask_path) in enumerate(zip(img_paths, mask_paths)):
            img_filename = os.path.splitext(os.path.basename(img_path))[0]
            mask_filename = os.path.splitext(os.path.basename(mask_path))[0]
            img = cv2.imread(img_path)
            mask = cv2.imread(mask_path)

            assert img_filename == mask_filename and img.shape[:2] == mask.shape[:2]

            k = 0
            for y in range(0, img.shape[0], TARGET_SIZE):
                for x in range(0, img.shape[1], TARGET_SIZE):
                    img_tile = img[y:y + TARGET_SIZE, x:x + TARGET_SIZE]
                    mask_tile = mask[y:y + TARGET_SIZE, x:x + TARGET_SIZE]

                    if img_tile.shape[0] == TARGET_SIZE and img_tile.shape[1] == TARGET_SIZE:
                        out_img_path = os.path.join(
                            OUTPUT_DIR, "images", "{}_{}.jpg".format(img_filename, k))
                        cv2.imwrite(out_img_path, img_tile)

                        out_mask_path = os.path.join(
                            OUTPUT_DIR, "masks", "{}_{}.png".format(mask_filename, k))
                        cv2.imwrite(out_mask_path, mask_tile)

                    k += 1

            print("Processed {} {}/{}".format(img_filename, i + 1, len(img_paths)))

    def download(self):
        """download and extract file.
        """
        # raise NotImplementedError
        file_url = posixpath.join(self.mirrors, self.resources)
        _urlretrieve(file_url, os.path.join(self.root, self.resources))

    def _check_exists(self):
        """Check file has been download or not"""
        # raise NotImplementedError
        self.data_path = os.path.join(
            self.root, "landcover",)

        return os.path.exists(os.path.join(self.data_path, "images")) and \
            os.path.exists(os.path.join(self.data_path, "masks"))

    def extract_file(self):
        """Extract file from compressed."""
        os.makedirs(os.path.join(self.root, "landcover"))
        shutil.unpack_archive(os.path.join(
            self.root, self.resources), os.path.join(self.root, "landcover"))
        os.remove(os.path.join(self.root, self.resources))
